/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.lang.reflect.InvocationTargetException;
import java.math.BigInteger;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer10;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.Textschluessel;
import org.jdtaus.banking.TextschluesselVerzeichnis;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.LogicalFile;
import org.jdtaus.banking.dtaus.LogicalFileType;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.banking.dtaus.PhysicalFileException;
import org.jdtaus.banking.dtaus.PhysicalFileFactory;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.banking.swing.AlphaNumericText27TextField;
import org.jdtaus.banking.swing.BankleitzahlTextField;
import org.jdtaus.banking.swing.KontonummerTextField;
import org.jdtaus.banking.swing.TextschluesselRenderer;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ImplementationException;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.io.FileOperations;
import org.jdtaus.core.io.util.RandomAccessFileOperations;
import org.jdtaus.core.monitor.Task;
import org.jdtaus.core.monitor.spi.TaskMonitor;
import org.jdtaus.core.text.Message;

/**
 * Dialog for generating a DTAUS test file.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: GeneratorDialog.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
public class GeneratorDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(GeneratorDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("descriptionPlaceholder");
        this._descriptionPlaceholder = (java.lang.String) p.getValue();


        p = meta.getProperty("maxDescriptions");
        this._maxDescriptions = ((java.lang.Integer) p.getValue()).intValue();


        p = meta.getProperty("maxTransactions");
        this._maxTransactions = ((java.lang.Integer) p.getValue()).intValue();

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        this.assertValidProperties();

        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency4;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency4 != null)
        {
           ret = this._dependency4;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency4 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency3;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency3 != null)
        {
           ret = this._dependency3;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TaskMonitor</code> implementation. */
    private transient TaskMonitor _dependency2;

    /**
     * Gets the configured <code>TaskMonitor</code> implementation.
     *
     * @return the configured <code>TaskMonitor</code> implementation.
     */
    private TaskMonitor getTaskMonitor()
    {
        TaskMonitor ret = null;
        if(this._dependency2 != null)
        {
           ret = this._dependency2;
        }
        else
        {
            ret = (TaskMonitor) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "TaskMonitor");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("TaskMonitor").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TextschluesselVerzeichnis</code> implementation. */
    private transient TextschluesselVerzeichnis _dependency1;

    /**
     * Gets the configured <code>TextschluesselVerzeichnis</code> implementation.
     *
     * @return the configured <code>TextschluesselVerzeichnis</code> implementation.
     */
    private TextschluesselVerzeichnis getTextschluesselVerzeichnis()
    {
        TextschluesselVerzeichnis ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (TextschluesselVerzeichnis) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "TextschluesselVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("TextschluesselVerzeichnis").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>PhysicalFileFactory</code> implementation. */
    private transient PhysicalFileFactory _dependency0;

    /**
     * Gets the configured <code>PhysicalFileFactory</code> implementation.
     *
     * @return the configured <code>PhysicalFileFactory</code> implementation.
     */
    private PhysicalFileFactory getPhysicalFileFactory()
    {
        PhysicalFileFactory ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (PhysicalFileFactory) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "PhysicalFileFactory");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("PhysicalFileFactory").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Property {@code descriptionPlaceholder}.
     * @serial
     */
    private java.lang.String _descriptionPlaceholder;

    /**
     * Gets the value of property <code>descriptionPlaceholder</code>.
     *
     * @return the value of property <code>descriptionPlaceholder</code>.
     */
    protected java.lang.String getDescriptionPlaceholder()
    {
        return this._descriptionPlaceholder;
    }

    /**
     * Property {@code maxDescriptions}.
     * @serial
     */
    private int _maxDescriptions;

    /**
     * Gets the value of property <code>maxDescriptions</code>.
     *
     * @return the value of property <code>maxDescriptions</code>.
     */
    protected int getMaxDescriptions()
    {
        return this._maxDescriptions;
    }

    /**
     * Property {@code maxTransactions}.
     * @serial
     */
    private int _maxTransactions;

    /**
     * Gets the value of property <code>maxTransactions</code>.
     *
     * @return the value of property <code>maxTransactions</code>.
     */
    protected int getMaxTransactions()
    {
        return this._maxTransactions;
    }


    //--------------------------------------------------------------Properties--
    //--GeneratorDialog---------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /** {@code Task} for generating physical files. */
    public static class GeneratorTask extends Task
    {
        /** Description message of the task. */
        public static final class Description extends Message
        {
            /** Empty format arguments. */
            private final Object[] formatArguments = {};

            public Object[] getFormatArguments(final Locale locale)
            {
                return this.formatArguments;
            }

            public String getText(final Locale locale)
            {
                return GeneratorDialogBundle.
                    getGeneratorTaskDescriptionText(locale);

            }
        }

        /**
         * Description of the task.
         * @serial
         */
        private Message description;

        public Message getDescription()
        {
            if(this.description == null)
            {
                this.description = new GeneratorTask.Description();
            }

            return this.description;
        }
    }

    /**
     * Creates new {@code GeneratorDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public GeneratorDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initializeProperties(GeneratorDialog.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the last generated file.
     *
     * @return the last generated file or {@code null} if no file has been
     * generated.
     */
    public File getGeneratedFile()
    {
        return this.generatedFile;
    }

    /**
     * Flag indicating if an asynchronous operation is in progress.
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    public boolean isWorking()
    {
        return this.working;
    }

    /**
     * Setter for property {@code working}.
     * <p>This method will disable the dialog's components when {@code working}
     * is {@code true} and should be used during asynchronous operations to
     * disable the dialog without closing it.</p>
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    protected void setWorking(final boolean working)
    {
        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {
                public void run()
                {
                    setCursor(Cursor.getPredefinedCursor(working ?
                        Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

                    closeButton.setEnabled(!working);
                    executiveAccountTextField.setEnabled(!working);
                    executiveBankTextField.setEnabled(!working);
                    executiveNameTextField.setEnabled(!working);
                    fileTypeList.setEnabled(!working);
                    generateButton.setEnabled(!working);
                    headerAccountTextField.setEnabled(!working);
                    headerBankTextField.setEnabled(!working);
                    headerCustomerTextField.setEnabled(!working);
                    numDescriptionsTextField.setEnabled(!working);
                    numFilesTextField.setEnabled(!working);
                    numTransactionsTextField.setEnabled(!working);
                    randomDescriptionCheckBox.setEnabled(!working);
                    searchExecutiveBankleitzahlButton.setEnabled(!working);
                    searchHeaderBankleitzahlButton.setEnabled(!working);
                    searchTargetBankleitzahlButton.setEnabled(!working);
                    targetAccountTextField.setEnabled(!working);
                    targetBankTextField.setEnabled(!working);
                    targetNameTextField.setEnabled(!working);
                    textschluesselList.setEnabled(!working);
                    GeneratorDialog.this.working = working;
                }
            });
        }
        catch(InterruptedException e)
        {
            throw new ImplementationException(GeneratorDialog.META, e);
        }
        catch(InvocationTargetException e)
        {
            throw new ImplementationException(GeneratorDialog.META,
                e.getTargetException() == null ? e : e.getTargetException());

        }
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    protected void assertValidProperties()
    {
        if(this.getMaxTransactions() < 0)
        {
            throw new PropertyException("maxTransactions",
                Integer.toString(this.getMaxTransactions()));

        }
        if(this.getMaxDescriptions() < 0)
        {
            throw new PropertyException("maxDescriptions",
                Integer.toString(this.getMaxDescriptions()));

        }
        if(this.getDescriptionPlaceholder() == null ||
            this.getDescriptionPlaceholder().length() <= 0)
        {
            throw new PropertyException("descriptionPlaceholder",
                this.getDescriptionPlaceholder());

        }
        try
        {
            AlphaNumericText27.parse(this.getDescriptionPlaceholder());
        }
        catch(ParseException e)
        {
            throw new PropertyException("descriptionPlaceholder",
                this.getDescriptionPlaceholder(), e);

        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        generateButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        executivePanel = new javax.swing.JPanel();
        executiveNameLabel = new javax.swing.JLabel();
        executiveNameTextField = new AlphaNumericText27TextField();
        executiveBankLabel = new javax.swing.JLabel();
        executiveBankTextField = new BankleitzahlTextField();
        executiveAccountLabel = new javax.swing.JLabel();
        executiveAccountTextField = new KontonummerTextField();
        searchExecutiveBankleitzahlButton = new javax.swing.JButton();
        targetPanel = new javax.swing.JPanel();
        targetNameLabel = new javax.swing.JLabel();
        targetNameTextField = new AlphaNumericText27TextField();
        targetBankLabel = new javax.swing.JLabel();
        targetBankTextField = new BankleitzahlTextField();
        targetAccountLabel = new javax.swing.JLabel();
        targetAccountTextField = new KontonummerTextField();
        searchTargetBankleitzahlButton = new javax.swing.JButton();
        textschluesselPanel = new javax.swing.JPanel();
        textschluesselScrollPane = new javax.swing.JScrollPane();
        textschluesselList = new javax.swing.JList();
        fileTypePanel = new javax.swing.JPanel();
        fileTypeScrollPane = new javax.swing.JScrollPane();
        fileTypeList = new javax.swing.JList();
        detailsPanel = new javax.swing.JPanel();
        numFilesLabel = new javax.swing.JLabel();
        numFilesTextField = new javax.swing.JFormattedTextField();
        numTransactionsLabel = new javax.swing.JLabel();
        numTransactionsTextField = new javax.swing.JFormattedTextField();
        numDescriptionsLabel = new javax.swing.JLabel();
        numDescriptionsTextField = new javax.swing.JFormattedTextField();
        randomDescriptionCheckBox = new javax.swing.JCheckBox();
        headerCustomerLabel = new javax.swing.JLabel();
        headerCustomerTextField = new AlphaNumericText27TextField();
        headerBankLabel = new javax.swing.JLabel();
        headerBankTextField = new BankleitzahlTextField();
        headerAccountLabel = new javax.swing.JLabel();
        headerAccountTextField = new KontonummerTextField();
        searchHeaderBankleitzahlButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(GeneratorDialogBundle.getTitleText(Locale.getDefault()));
        setName("Generator Dialog");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        generateButton.setText(GeneratorDialogBundle.getGenerateButtonText(Locale.getDefault()));
        generateButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                generateButtonActionPerformed(evt);
            }
        });

        closeButton.setText(GeneratorDialogBundle.getCloseButtonText(Locale.getDefault()));
        closeButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeButtonActionPerformed(evt);
            }
        });

        executivePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getExecutivePanelTitleText(Locale.getDefault())));
        executiveNameLabel.setText(GeneratorDialogBundle.getExecutiveNameText(Locale.getDefault()));

        executiveNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveBankLabel.setText(GeneratorDialogBundle.getExecutiveBankText(Locale.getDefault()));

        executiveBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        executiveAccountLabel.setText(GeneratorDialogBundle.getExecutiveAccountText(Locale.getDefault()));

        executiveAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchExecutiveBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.executiveBankTextField, true));
        searchExecutiveBankleitzahlButton.setBorderPainted(false);
        searchExecutiveBankleitzahlButton.setContentAreaFilled(false);
        searchExecutiveBankleitzahlButton.setDefaultCapable(false);
        searchExecutiveBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchExecutiveBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchExecutiveBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout executivePanelLayout = new org.jdesktop.layout.GroupLayout(executivePanel);
        executivePanel.setLayout(executivePanelLayout);
        executivePanelLayout.setHorizontalGroup(
            executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(executivePanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(executiveAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executiveNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executivePanelLayout.createSequentialGroup()
                        .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 234, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchExecutiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executiveNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 256, Short.MAX_VALUE))
                .addContainerGap())
        );
        executivePanelLayout.setVerticalGroup(
            executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(executivePanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveNameLabel)
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(searchExecutiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 17, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveAccountLabel))
                .addContainerGap(34, Short.MAX_VALUE))
        );

        targetPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getTargetPanelTitleText(Locale.getDefault())));
        targetNameLabel.setText(GeneratorDialogBundle.getTargetNameText(Locale.getDefault()));

        targetNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetBankLabel.setText(GeneratorDialogBundle.getTargetBankText(Locale.getDefault()));

        targetBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        targetAccountLabel.setText(GeneratorDialogBundle.getTargetAccountText(Locale.getDefault()));

        targetAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchTargetBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.targetBankTextField, true));
        searchTargetBankleitzahlButton.setBorderPainted(false);
        searchTargetBankleitzahlButton.setContentAreaFilled(false);
        searchTargetBankleitzahlButton.setDefaultCapable(false);
        searchTargetBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchTargetBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchTargetBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout targetPanelLayout = new org.jdesktop.layout.GroupLayout(targetPanel);
        targetPanel.setLayout(targetPanelLayout);
        targetPanelLayout.setHorizontalGroup(
            targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(targetPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(targetAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(targetBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(targetNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, targetPanelLayout.createSequentialGroup()
                        .add(targetBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 235, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchTargetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 257, Short.MAX_VALUE)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 257, Short.MAX_VALUE))
                .addContainerGap())
        );
        targetPanelLayout.setVerticalGroup(
            targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(targetPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetNameLabel)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(searchTargetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targetBankLabel)
                    .add(targetBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetAccountLabel)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(34, Short.MAX_VALUE))
        );

        textschluesselPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getTextschluesselPanelTitleText(Locale.getDefault())));
        textschluesselList.setModel(new DefaultListModel());
        textschluesselList.setCellRenderer(new TextschluesselRenderer());
        for(Textschluessel key : this.getSupportedTextschluessel())
        {
            ((DefaultListModel) this.textschluesselList.getModel()).addElement(key);
        }
        textschluesselScrollPane.setViewportView(textschluesselList);

        org.jdesktop.layout.GroupLayout textschluesselPanelLayout = new org.jdesktop.layout.GroupLayout(textschluesselPanel);
        textschluesselPanel.setLayout(textschluesselPanelLayout);
        textschluesselPanelLayout.setHorizontalGroup(
            textschluesselPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, textschluesselScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 377, Short.MAX_VALUE)
        );
        textschluesselPanelLayout.setVerticalGroup(
            textschluesselPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(textschluesselScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 114, Short.MAX_VALUE)
        );

        fileTypePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getFileTypePanelTitleText(Locale.getDefault())));
        fileTypeList.setModel(new DefaultListModel());
        fileTypeList.setCellRenderer(new LogicalFileTypeRenderer());
        for(LogicalFileType type : this.getSupportedFileTypes())
        {
            ((DefaultListModel) this.fileTypeList.getModel()).addElement(type);
        }
        fileTypeList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
        {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt)
            {
                fileTypeListValueChanged(evt);
            }
        });

        fileTypeScrollPane.setViewportView(fileTypeList);

        org.jdesktop.layout.GroupLayout fileTypePanelLayout = new org.jdesktop.layout.GroupLayout(fileTypePanel);
        fileTypePanel.setLayout(fileTypePanelLayout);
        fileTypePanelLayout.setHorizontalGroup(
            fileTypePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(fileTypeScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE)
        );
        fileTypePanelLayout.setVerticalGroup(
            fileTypePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(fileTypeScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 114, Short.MAX_VALUE)
        );

        detailsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getDetailsPanelTitleText(Locale.getDefault())));
        numFilesLabel.setText(GeneratorDialogBundle.getNumFilesText(Locale.getDefault()));

        numFilesTextField.setColumns(20);
        numFilesTextField.setValue(new Long(0L));

        numTransactionsLabel.setText(GeneratorDialogBundle.getNumTransactionsText(Locale.getDefault()));

        numTransactionsTextField.setColumns(7);
        numTransactionsTextField.setValue(new Integer(0));

        numDescriptionsLabel.setText(GeneratorDialogBundle.getNumDescriptionsText(Locale.getDefault()));

        numDescriptionsTextField.setColumns(2);
        numDescriptionsTextField.setValue(new Integer(0));

        randomDescriptionCheckBox.setText(GeneratorDialogBundle.getVariableDescriptionText(Locale.getDefault()));
        randomDescriptionCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        randomDescriptionCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        randomDescriptionCheckBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                randomDescriptionCheckBoxActionPerformed(evt);
                randomDescriptionCheckBoxActionPerformed1(evt);
            }
        });

        headerCustomerLabel.setText(GeneratorDialogBundle.getHeaderCustomerText(Locale.getDefault()));

        headerCustomerTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        headerBankLabel.setText(GeneratorDialogBundle.getHeaderBankText(Locale.getDefault()));

        headerBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        headerAccountLabel.setText(GeneratorDialogBundle.getHeaderAccountText(Locale.getDefault()));

        headerAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchHeaderBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.headerBankTextField, true));
        searchHeaderBankleitzahlButton.setBorderPainted(false);
        searchHeaderBankleitzahlButton.setContentAreaFilled(false);
        searchHeaderBankleitzahlButton.setDefaultCapable(false);
        searchHeaderBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchHeaderBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchHeaderBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout detailsPanelLayout = new org.jdesktop.layout.GroupLayout(detailsPanel);
        detailsPanel.setLayout(detailsPanelLayout);
        detailsPanelLayout.setHorizontalGroup(
            detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(detailsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(detailsPanelLayout.createSequentialGroup()
                        .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(headerCustomerLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(numDescriptionsLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(numTransactionsLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(numFilesLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(headerAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED))
                    .add(detailsPanelLayout.createSequentialGroup()
                        .add(headerBankLabel)
                        .add(289, 289, 289)))
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(headerAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(numDescriptionsTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 173, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(randomDescriptionCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 193, Short.MAX_VALUE))
                    .add(numTransactionsTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(numFilesTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(headerCustomerTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(headerBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchHeaderBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        detailsPanelLayout.setVerticalGroup(
            detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(detailsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numFilesLabel)
                    .add(numFilesTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numTransactionsLabel)
                    .add(numTransactionsTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numDescriptionsLabel)
                    .add(randomDescriptionCheckBox)
                    .add(numDescriptionsTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerCustomerTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerCustomerLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerBankLabel)
                    .add(searchHeaderBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerAccountLabel))
                .addContainerGap(84, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, detailsPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(generateButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(closeButton))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(fileTypePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(executivePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(textschluesselPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(targetPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(targetPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executivePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(fileTypePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(textschluesselPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(7, 7, 7)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(generateButton)
                    .add(closeButton))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void fileTypeListValueChanged(javax.swing.event.ListSelectionEvent evt)//GEN-FIRST:event_fileTypeListValueChanged
    {//GEN-HEADEREND:event_fileTypeListValueChanged
        final Object[] selectedTypes = this.fileTypeList.getSelectedValues();
        final Object[] prevSelection = this.textschluesselList.
            getSelectedValues();

        this.getTextschluesselModel().clear();

        if(selectedTypes == null || selectedTypes.length == 0)
        {
            for(Textschluessel key : this.getSupportedTextschluessel())
            {
                this.getTextschluesselModel().addElement(key);
            }
        }
        else
        {
            boolean debitAllowed = false;
            boolean remittanceAllowed = false;

            for(int i = selectedTypes.length - 1; i >= 0; i--)
            {
                if(!debitAllowed)
                {
                    debitAllowed = ((LogicalFileType) selectedTypes[i]).
                        isDebitAllowed();

                }
                if(!remittanceAllowed)
                {
                    remittanceAllowed = ((LogicalFileType) selectedTypes[i]).
                        isRemittanceAllowed();

                }

                if(debitAllowed && remittanceAllowed)
                {
                    break;
                }
            }

            final ArrayList<Textschluessel> keys =
                new ArrayList<Textschluessel>(this.getSupportedTextschluessel().
                size());

            if(debitAllowed)
            {
                keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
                    search(true, false)));

                if(remittanceAllowed)
                {
                    keys.addAll(Arrays.asList(getTextschluesselVerzeichnis().
                        search(true, true)));

                }
            }

            if(remittanceAllowed)
            {
                keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
                    search(false, true)));

                if(debitAllowed)
                {
                    keys.addAll(Arrays.asList(getTextschluesselVerzeichnis().
                        search(true, true)));

                }
            }

            for(Textschluessel key : keys)
            {
                this.getTextschluesselModel().addElement(key);
            }
        }

        if(prevSelection != null)
        {
            final List<Integer> indicesList =
                new ArrayList<Integer>(prevSelection.length);

            for(int i = prevSelection.length - 1; i >= 0; i--)
            {
                final int index = this.getTextschluesselModel().
                    indexOf(prevSelection[i]);

                if(index >= 0)
                {
                    indicesList.add(index);
                }
            }

            int i = 0;
            final int[] indices = new int[indicesList.size()];
            for(Integer index : indicesList)
            {
                indices[i++] = index;
            }

            this.textschluesselList.setSelectedIndices(indices);
        }
    }//GEN-LAST:event_fileTypeListValueChanged

    private void generateButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_generateButtonActionPerformed
    {//GEN-HEADEREND:event_generateButtonActionPerformed
        this.doGenerate();
    }//GEN-LAST:event_generateButtonActionPerformed

    private void randomDescriptionCheckBoxActionPerformed1(java.awt.event.ActionEvent evt)//GEN-FIRST:event_randomDescriptionCheckBoxActionPerformed1
    {//GEN-HEADEREND:event_randomDescriptionCheckBoxActionPerformed1
        numDescriptionsTextField.setValue(null);
    }//GEN-LAST:event_randomDescriptionCheckBoxActionPerformed1

    private void randomDescriptionCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_randomDescriptionCheckBoxActionPerformed
    {//GEN-HEADEREND:event_randomDescriptionCheckBoxActionPerformed
        numDescriptionsTextField.setEnabled(!this.randomDescriptionCheckBox.isSelected());
    }//GEN-LAST:event_randomDescriptionCheckBoxActionPerformed

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
}//GEN-LAST:event_closeButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        if(!this.isWorking())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(final int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void doGenerate()
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                String str;
                boolean valid = true;

                AlphaNumericText27 headerCustomer;
                AlphaNumericText27 executiveName;
                AlphaNumericText27 targetName;
                Bankleitzahl headerBank;
                Bankleitzahl executiveBank;
                Bankleitzahl targetBank;
                Kontonummer headerAccount;
                Kontonummer executiveAccount;
                Kontonummer targetAccount;
                int numDescriptions = -1;
                long numFiles = -1L;
                int numTransactions = -1;
                final boolean random = randomDescriptionCheckBox.isSelected();
                final Object[] selectedFileTypes = fileTypeList.
                    getSelectedValues();

                final Object[] selectedTextschluessel = textschluesselList.
                    getSelectedValues();

                valid = selectedFileTypes != null &&
                    selectedTextschluessel != null;

                try
                {
                    str = headerCustomerTextField.getText();
                    headerCustomer = AlphaNumericText27.parse(str);
                    resetForeground(headerCustomerLabel);
                }
                catch(ParseException e)
                {
                    setForeground(headerCustomerLabel, Color.RED);
                    headerCustomerTextField.setText(AlphaNumericText27.
                        normalize(headerCustomerTextField.getText()));

                    valid = false;
                    headerCustomer = null;
                }

                try
                {
                    str = executiveNameTextField.getText();
                    executiveName = AlphaNumericText27.parse(str);
                    resetForeground(executiveNameLabel);
                }
                catch(ParseException e)
                {
                    setForeground(executiveNameLabel, Color.RED);
                    executiveNameTextField.setText(AlphaNumericText27.
                        normalize(executiveNameTextField.getText()));

                    valid = false;
                    executiveName = null;
                }

                try
                {
                    str = targetNameTextField.getText();
                    targetName = AlphaNumericText27.parse(str);
                    resetForeground(targetNameLabel);
                }
                catch(ParseException e)
                {
                    setForeground(targetNameLabel, Color.RED);
                    targetNameTextField.setText(AlphaNumericText27.
                        normalize(targetNameTextField.getText()));

                    valid = false;
                    targetName = null;
                }

                try
                {
                    str = headerBankTextField.getText();
                    headerBank = Bankleitzahl.parse(str);
                    resetForeground(headerBankLabel);
                }
                catch(ParseException e)
                {
                    setForeground(headerBankLabel, Color.RED);
                    valid = false;
                    headerBank = null;
                }

                try
                {
                    str = executiveBankTextField.getText();
                    executiveBank = Bankleitzahl.parse(str);
                    resetForeground(executiveBankLabel);
                }
                catch(ParseException e)
                {
                    setForeground(executiveBankLabel, Color.RED);
                    valid = false;
                    executiveBank = null;
                }

                try
                {
                    str = targetBankTextField.getText();
                    targetBank = Bankleitzahl.parse(str);
                    resetForeground(targetBankLabel);
                }
                catch(ParseException e)
                {
                    setForeground(targetBankLabel, Color.RED);
                    valid = false;
                    targetBank = null;
                }

                try
                {
                    str = headerAccountTextField.getText();
                    headerAccount = Kontonummer.parse(str);
                    resetForeground(headerAccountLabel);
                }
                catch(ParseException e)
                {
                    setForeground(headerAccountLabel, Color.RED);
                    valid = false;
                    headerAccount = null;
                }

                try
                {
                    str = executiveAccountTextField.getText();
                    executiveAccount = Kontonummer.parse(str);
                    resetForeground(executiveAccountLabel);
                }
                catch(ParseException e)
                {
                    setForeground(executiveAccountLabel, Color.RED);
                    valid = false;
                    executiveAccount = null;
                }

                try
                {
                    str = targetAccountTextField.getText();
                    targetAccount = Kontonummer.parse(str);
                    resetForeground(targetAccountLabel);
                }
                catch(ParseException e)
                {
                    setForeground(targetAccountLabel, Color.RED);
                    valid = false;
                    targetAccount = null;
                }

                if(numFilesTextField.getValue() != null)
                {
                    numFiles = ((Number) numFilesTextField.getValue()).
                        longValue();

                    if(numFiles <= 0 || selectedFileTypes.length <= 0 ||
                        (numFiles > 0 && selectedFileTypes.length > numFiles))
                    {
                        setForeground(numFilesLabel, Color.RED);
                        valid = false;

                        if(numFiles > 0 && (selectedFileTypes.length <= 0 ||
                            selectedFileTypes.length > numFiles))
                        {
                            setForeground(fileTypeList, Color.RED);
                        }
                        else
                        {
                            resetForeground(fileTypeList);
                        }
                    }
                    else
                    {
                        resetForeground(numFilesLabel);
                        resetForeground(fileTypeList);
                    }
                }
                else
                {
                    setForeground(numFilesLabel, Color.RED);
                    valid = false;
                }

                if(numTransactionsTextField.getValue() != null)
                {
                    numTransactions =
                        ((Number) numTransactionsTextField.getValue()).
                        intValue();

                    if(numTransactions < 0
                        || numTransactions > getMaxTransactions()
                        || (numTransactions > 0
                        && (selectedTextschluessel.length <= 0
                        || selectedTextschluessel.length > numTransactions)))
                    {
                        setForeground(numTransactionsLabel, Color.RED);
                        valid = false;

                        if(numTransactions > 0 &&
                            (selectedTextschluessel.length <= 0 ||
                            selectedTextschluessel.length > numTransactions))
                        {
                            setForeground(textschluesselList, Color.RED);
                        }
                        else
                        {
                            resetForeground(textschluesselList);
                        }
                    }
                    else
                    {
                        resetForeground(numTransactionsLabel);
                        resetForeground(textschluesselList);
                    }
                }
                else
                {
                    setForeground(numTransactionsLabel, Color.RED);
                    valid = false;
                }

                if(!random)
                {
                    if(numDescriptionsTextField.getValue() != null)
                    {
                        numDescriptions =
                            ((Number) numDescriptionsTextField.getValue()).
                            intValue();

                        if(numDescriptions < 0
                            || numDescriptions > getMaxDescriptions())
                        {
                            setForeground(numDescriptionsLabel, Color.RED);
                            valid = false;
                        }
                        else
                        {
                            resetForeground(numDescriptionsLabel);
                        }
                    }
                    else
                    {
                        setForeground(numDescriptionsLabel, Color.RED);
                        valid = false;
                        numDescriptions = -1;
                    }
                }
                else
                {
                    numDescriptions = -1;
                }

                if(valid)
                {
                    final JFileChooser chooser = new JFileChooser();
                    if(chooser.showSaveDialog(
                        GeneratorDialog.this) == JFileChooser.APPROVE_OPTION &&
                        chooser.getSelectedFile() != null)
                    {
                        generatedFile = chooser.getSelectedFile();
                        generate(generatedFile, headerCustomer,
                            headerBank, headerAccount, executiveName,
                            executiveBank, executiveAccount, targetName,
                            targetBank, targetAccount, selectedFileTypes,
                            selectedTextschluessel, numFiles, numTransactions,
                            random ? -1 : numDescriptions);

                    }
                }
                else
                {
                    UIManager.getLookAndFeel().provideErrorFeedback(
                        getUserInterface().getParent());

                }
            }
        });
    }

    private List<? extends LogicalFileType> getSupportedFileTypes()
    {
        final List<LogicalFileType> types =
            new ArrayList<LogicalFileType>(50);

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, true, true)));

        return types;
    }

    private List<? extends Textschluessel> getSupportedTextschluessel()
    {
        final List<Textschluessel> keys = new ArrayList<Textschluessel>(50);

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(false, false)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(true, false)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(false, true)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(true, true)));

        return keys;
    }

    private void setForeground(final JComponent comp, final Color color)
    {
        if(!this.colors.containsKey(comp))
        {
            this.colors.put(comp, comp.getForeground());
        }
        comp.setForeground(color);
    }

    private void resetForeground(final JComponent comp)
    {
        if(this.colors.containsKey(comp))
        {
            comp.setForeground(this.colors.get(comp));
        }
    }

    private void onError(final Throwable t, final Object messages)
    {
        UIManager.getLookAndFeel().provideErrorFeedback(
            getUserInterface().getParent());

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final String name = t.getClass().getName();
                JOptionPane.showMessageDialog(GeneratorDialog.this, messages,
                    name.substring(name.lastIndexOf('.') + 1),
                    JOptionPane.ERROR_MESSAGE);

            }
        });
    }

    private DefaultListModel getTextschluesselModel()
    {
        return (DefaultListModel) this.textschluesselList.getModel();
    }

    private void generate(final File file,
        final AlphaNumericText27 customer,
        final Bankleitzahl bank, final Kontonummer account,
        final AlphaNumericText27 executiveName,
        final Bankleitzahl executiveBank, final Kontonummer executiveAccount,
        final AlphaNumericText27 targetName, final Bankleitzahl targetBank,
        final Kontonummer targetAccount, final Object[] fileTypes,
        final Object[] textschluessel, final long numFiles,
        final int numTransactions, final int numDescriptions)
    {
        final Header h = new Header();
        final Transaction t = new Transaction();
        final int[] random = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14};
        final Task task = new GeneratorTask();

        h.setAccount(account);
        h.setBank(bank);
        h.setBankData(null);
        h.setCurrency(Currency.getInstance("EUR"));
        h.setCustomer(customer);
        h.setSchedule(new Header.Schedule(new Date(), new Date()));
        t.setAmount(BigInteger.valueOf(1L));
        t.setCurrency(Currency.getInstance("EUR"));
        t.setExecutiveAccount(executiveAccount);
        t.setExecutiveBank(executiveBank);
        t.setExecutiveExt(executiveName);
        t.setExecutiveName(executiveName);
        t.setPrimaryBank(null);
        t.setTargetAccount(targetAccount);
        t.setTargetBank(targetBank);
        t.setTargetExt(targetName);
        t.setTargetName(targetName);

        try
        {
            if(numDescriptions > 0)
            {
                final AlphaNumericText27 dsc = executiveName.isEmpty()
                ? AlphaNumericText27.parse(this.getDescriptionPlaceholder())
                : executiveName;

                for(int d = numDescriptions; d > 0; d--)
                {
                    t.getDescription().addDescription(dsc);
                }
            }
        }
        catch(ParseException e)
        {
            throw new ImplementationException(GeneratorDialog.META, e);
        }

        task.setIndeterminate(false);
        task.setMaximum(numFiles * numTransactions > Integer.MAX_VALUE ?
            Integer.MAX_VALUE : (int) (numFiles * numTransactions));

        task.setMinimum(0);
        task.setProgress(0);

        final Thread th = new Thread()
        {
            public void run()
            {
                try
                {
                    LogicalFile lFile;
                    final PhysicalFile pFile;
                    final FileOperations ops = new RandomAccessFileOperations(
                        new RandomAccessFile(file, "rw"));

                    int progress = 0;
                    int typeIndex = fileTypes.length - 1;
                    int keyIndex = textschluessel.length - 1;
                    int randomIndex = random.length - 1;

                    setWorking(true);
                    ops.setLength(0L);
                    pFile = getPhysicalFileFactory().getPhysicalFile(ops);
                    getTaskMonitor().monitor(task);

                    for(long i = numFiles; i > 0L; i--)
                    {
                        final LogicalFileType type =
                            (LogicalFileType) fileTypes[typeIndex];

                        if(--typeIndex < 0)
                        {
                            typeIndex = fileTypes.length - 1;
                        }

                        h.setReference(Referenznummer10.valueOf(
                            Long.valueOf(i)));

                        h.setType(type);

                        if(type.isSendByBank())
                        {
                            h.setBankData(executiveBank);
                        }
                        else
                        {
                            h.setBankData(null);
                        }

                        lFile = pFile.add(h);
                        for(int j = numTransactions; j > 0; j--)
                        {
                            if(++progress > Integer.MAX_VALUE)
                            {
                                progress = Integer.MAX_VALUE;
                            }
                            task.setProgress(progress);

                            t.setReference(Referenznummer11.valueOf(
                                Long.valueOf(j)));

                            Textschluessel txtKey = null;
                            boolean cycle = false;

                            do
                            {
                                txtKey = (Textschluessel)
                                textschluessel[keyIndex];

                                if(--keyIndex < 0)
                                {
                                    keyIndex = textschluessel.length - 1;

                                    if(cycle)
                                    {
                                        txtKey = null;
                                        break;
                                    }
                                    else
                                    {
                                        cycle = true;
                                    }
                                }
                            } while(
                                (type.isDebitAllowed() && !txtKey.isDebit()) ||
                                (type.isRemittanceAllowed() &&
                                !txtKey.isRemittance()));

                            if(txtKey == null)
                            {
                                continue;
                            }

                            t.setType(txtKey);

                            if(numDescriptions < 0)
                            {
                                t.setDescription(
                                    new Transaction.Description());

                                for(int d = random[randomIndex]; d > 0; d--)
                                {
                                    t.getDescription().
                                        addDescription(executiveName);

                                }

                                if(--randomIndex < 0)
                                {
                                    randomIndex = random.length - 1;
                                }
                            }

                            lFile.createTransaction(t);
                        }
                    }
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                catch(PhysicalFileException e)
                {
                    final Object[] messages =
                        new Object[e.getMessages().length];

                    for(int i = 0; i < messages.length && i < 25; i++)
                    {
                        messages[i] = e.getMessages()[i].
                            getText(Locale.getDefault());

                    }

                    onError(e, messages);
                }
                finally
                {
                    getTaskMonitor().finish(task);
                    setWorking(false);
                }
            }
        };

        th.setUncaughtExceptionHandler(this.getExceptionHandler());
        th.start();
    }

    private Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        return this.getApplication().getExceptionHandler();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel detailsPanel;
    private javax.swing.JLabel executiveAccountLabel;
    private javax.swing.JTextField executiveAccountTextField;
    private javax.swing.JLabel executiveBankLabel;
    private javax.swing.JTextField executiveBankTextField;
    private javax.swing.JLabel executiveNameLabel;
    private javax.swing.JTextField executiveNameTextField;
    private javax.swing.JPanel executivePanel;
    private javax.swing.JList fileTypeList;
    private javax.swing.JPanel fileTypePanel;
    private javax.swing.JScrollPane fileTypeScrollPane;
    private javax.swing.JButton generateButton;
    private javax.swing.JLabel headerAccountLabel;
    private javax.swing.JTextField headerAccountTextField;
    private javax.swing.JLabel headerBankLabel;
    private javax.swing.JTextField headerBankTextField;
    private javax.swing.JLabel headerCustomerLabel;
    private javax.swing.JTextField headerCustomerTextField;
    private javax.swing.JLabel numDescriptionsLabel;
    private javax.swing.JFormattedTextField numDescriptionsTextField;
    private javax.swing.JLabel numFilesLabel;
    private javax.swing.JFormattedTextField numFilesTextField;
    private javax.swing.JLabel numTransactionsLabel;
    private javax.swing.JFormattedTextField numTransactionsTextField;
    private javax.swing.JCheckBox randomDescriptionCheckBox;
    private javax.swing.JButton searchExecutiveBankleitzahlButton;
    private javax.swing.JButton searchHeaderBankleitzahlButton;
    private javax.swing.JButton searchTargetBankleitzahlButton;
    private javax.swing.JLabel targetAccountLabel;
    private javax.swing.JTextField targetAccountTextField;
    private javax.swing.JLabel targetBankLabel;
    private javax.swing.JTextField targetBankTextField;
    private javax.swing.JLabel targetNameLabel;
    private javax.swing.JTextField targetNameTextField;
    private javax.swing.JPanel targetPanel;
    private javax.swing.JList textschluesselList;
    private javax.swing.JPanel textschluesselPanel;
    private javax.swing.JScrollPane textschluesselScrollPane;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JComponent, Color> colors = new HashMap(50);

    /**
     * Flag indicating if an asynchronous operation is in progress.
     * @serial
     */
    private boolean working;

    /**
     * Last generated file or {@code null}.
     */
    private transient File generatedFile;

    //---------------------------------------------------------GeneratorDialog--

}
