/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import javax.swing.AbstractListModel;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.LogicalFile;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.banking.dtaus.PhysicalFileException;
import org.jdtaus.banking.dtaus.PhysicalFileFactory;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ImplementationException;
import org.jdtaus.core.container.Implementations;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.io.FileOperations;
import org.jdtaus.core.io.util.RandomAccessFileOperations;
import org.jdtaus.editor.IconSet;
import org.jdtaus.editor.PhysicalFileFormatter;

/**
 * Frame to display and edit {@code PhysicalFile} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
public class PhysicalFileFrame extends JFrame implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(PhysicalFileFrame.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        JFrame.setDefaultLookAndFeelDecorated(true);

        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>PhysicalFileTextFormatter</code> implementation. */
    private transient PhysicalFileFormatter _dependency4;

    /**
     * Gets the configured <code>PhysicalFileTextFormatter</code> implementation.
     *
     * @return the configured <code>PhysicalFileTextFormatter</code> implementation.
     */
    private PhysicalFileFormatter getPhysicalFileTextFormatter()
    {
        PhysicalFileFormatter ret = null;
        if(this._dependency4 != null)
        {
           ret = this._dependency4;
        }
        else
        {
            ret = (PhysicalFileFormatter) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "PhysicalFileTextFormatter");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("PhysicalFileTextFormatter").
                isBound())
            {
                this._dependency4 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency3;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency3 != null)
        {
           ret = this._dependency3;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
           ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>PhysicalFileFactory</code> implementation. */
    private transient PhysicalFileFactory _dependency1;

    /**
     * Gets the configured <code>PhysicalFileFactory</code> implementation.
     *
     * @return the configured <code>PhysicalFileFactory</code> implementation.
     */
    private PhysicalFileFactory getPhysicalFileFactory()
    {
        PhysicalFileFactory ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (PhysicalFileFactory) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "PhysicalFileFactory");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("PhysicalFileFactory").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>IconSet</code> implementation. */
    private transient IconSet _dependency0;

    /**
     * Gets the configured <code>IconSet</code> implementation.
     *
     * @return the configured <code>IconSet</code> implementation.
     */
    private IconSet getIconSet()
    {
        IconSet ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (IconSet) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "IconSet");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("IconSet").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //--------------------------------------------------------------Properties--
    //--PhysicalFileFrame-------------------------------------------------------

    /** Creates new form PhysicalFileFrame */
    public PhysicalFileFrame()
    {
        super();
        this.initializeProperties(PhysicalFileFrame.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the physical file of the editor.
     *
     * @return the physical file of the editor or {@code null}.
     */
    public PhysicalFile getPhysicalFile()
    {
        return this.pFile;
    }

    /**
     * Sets the physical file to edit.
     *
     * @param pFile the physical file for the editor or {@code null}.
     */
    public void setPhysicalFile(final PhysicalFile pFile)
    {
        this.firePropertyChange("physicalFile", this.pFile, pFile);
        this.pFile = pFile;
        this.updatePhysicalFile();
    }

    /**
     * Gets the logical file of the editor.
     *
     * @return the logical file of the editor or {@code null}.
     */
    public LogicalFile getLogicalFile()
    {
        return this.lFile;
    }

    /**
     * Sets the logical file of the editor.
     *
     * @param lFile the logical file of the editor or {@code null}.
     */
    protected void setLogicalFile(final LogicalFile lFile)
    {
        this.firePropertyChange("logicalFile", this.lFile, lFile);
        this.lFile = lFile;
        this.updateLogicalFile();
    }

    /**
     * Flag indicating if an asynchronous operation is in progress.
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    public boolean isWorking()
    {
        return this.working;
    }

    /**
     * Setter for property {@code working}.
     * <p>This method will disable the frame's components when {@code working}
     * is {@code true} and should be used during asynchronous operations to
     * disable the frame without closing it.</p>
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    protected void setWorking(final boolean working)
    {
        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {
                public void run()
                {
                    setCursor(Cursor.getPredefinedCursor(working ?
                        Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

                    logicalFileList.setEnabled(!working);
                    editMenu.setEnabled(!working);
                    fileMenu.setEnabled(!working);
                    toolsMenu.setEnabled(!working);
                    lafMenu.setEnabled(!working);
                    transactionTable.setEnabled(!working);
                    PhysicalFileFrame.this.working = working;
                }
            });
        }
        catch(InterruptedException e)
        {
            throw new ImplementationException(PhysicalFileFrame.META, e);
        }
        catch(InvocationTargetException e)
        {
            throw new ImplementationException(PhysicalFileFrame.META,
                e.getTargetException() == null ? e : e.getTargetException());

        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        logicalFilePanel = new javax.swing.JPanel();
        logicalFileScrollPane = new javax.swing.JScrollPane();
        logicalFileList = new javax.swing.JList();
        transactionPanel = new javax.swing.JPanel();
        transactionScrollPane = new javax.swing.JScrollPane();
        transactionTable = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        newMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        openMenuItem = new javax.swing.JMenuItem();
        closeMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        exitMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        addLogicalFileMenuItem = new javax.swing.JMenuItem();
        updateLogicalFileMenuItem = new javax.swing.JMenuItem();
        removeLogicalFileMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        addTransactionMenuItem = new javax.swing.JMenuItem();
        updateTransactionMenuItem = new javax.swing.JMenuItem();
        removeTransactionMenuItem = new javax.swing.JMenuItem();
        toolsMenu = new javax.swing.JMenu();
        generatorMenuItem = new javax.swing.JMenuItem();
        bankDirectoryMenuItem = new javax.swing.JMenuItem();
        lafMenu = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(PhysicalFileFrameBundle.getFrameTitleText(Locale.getDefault()));
        setIconImage(this.getIconSet().getIcon(IconSet.DEV_APPLICATION, IconSet.SIZE_16X16, Locale.getDefault()).getImage());
        setLocationByPlatform(true);
        setName("jDTAUS Editor");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                formWindowClosing(evt);
            }
            public void windowOpened(java.awt.event.WindowEvent evt)
            {
                formWindowOpened(evt);
            }
        });

        logicalFilePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(PhysicalFileFrameBundle.getLogicalFilePanelTitleText(Locale.getDefault())));
        logicalFileList.setModel(new PhysicalFileFrameListModel());
        logicalFileList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        logicalFileList.setCellRenderer(new LogicalFileRenderer());
        logicalFileList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
        {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt)
            {
                logicalFileListValueChanged(evt);
            }
        });

        logicalFileScrollPane.setViewportView(logicalFileList);

        org.jdesktop.layout.GroupLayout logicalFilePanelLayout = new org.jdesktop.layout.GroupLayout(logicalFilePanel);
        logicalFilePanel.setLayout(logicalFilePanelLayout);
        logicalFilePanelLayout.setHorizontalGroup(
            logicalFilePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(logicalFileScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 753, Short.MAX_VALUE)
        );
        logicalFilePanelLayout.setVerticalGroup(
            logicalFilePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(logicalFileScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 143, Short.MAX_VALUE)
        );

        transactionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(PhysicalFileFrameBundle.getTransactionPanelTitleText(Locale.getDefault())));
        transactionTable.setModel(new PhysicalFileFrameTableModel());
        transactionTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        transactionTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            public void valueChanged(final ListSelectionEvent event)
            {
                transactionTableValueChanged(event);
            }
        });

        transactionScrollPane.setViewportView(transactionTable);

        org.jdesktop.layout.GroupLayout transactionPanelLayout = new org.jdesktop.layout.GroupLayout(transactionPanel);
        transactionPanel.setLayout(transactionPanelLayout);
        transactionPanelLayout.setHorizontalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 753, Short.MAX_VALUE)
        );
        transactionPanelLayout.setVerticalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 491, Short.MAX_VALUE)
        );

        fileMenu.setText(PhysicalFileFrameBundle.getFileMenuText(Locale.getDefault()));
        newMenuItem.setIcon(this.getIconSet().getIcon(IconSet.NEW, IconSet.SIZE_16X16, Locale.getDefault()));
        newMenuItem.setText(PhysicalFileFrameBundle.getFileNewText(Locale.getDefault()));
        newMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(newMenuItem);

        fileMenu.add(jSeparator1);

        openMenuItem.setIcon(this.getIconSet().getIcon(IconSet.OPEN, IconSet.SIZE_16X16, Locale.getDefault()));
        openMenuItem.setText(PhysicalFileFrameBundle.getFileOpenText(Locale.getDefault()));
        openMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                openMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openMenuItem);

        closeMenuItem.setText(PhysicalFileFrameBundle.getFileCloseText(Locale.getDefault()));
        closeMenuItem.setEnabled(false);
        closeMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(closeMenuItem);

        saveAsMenuItem.setIcon(this.getIconSet().getIcon(IconSet.SAVE_AS, IconSet.SIZE_16X16, Locale.getDefault()));
        saveAsMenuItem.setText(PhysicalFileFrameBundle.getFileSaveAsText(Locale.getDefault()));
        saveAsMenuItem.setEnabled(false);
        saveAsMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                saveAsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveAsMenuItem);

        fileMenu.add(jSeparator2);

        exitMenuItem.setText(PhysicalFileFrameBundle.getFileExitText(Locale.getDefault()));
        exitMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        editMenu.setText(PhysicalFileFrameBundle.getEditMenuText(Locale.getDefault()));
        addLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.ADD, IconSet.SIZE_16X16, Locale.getDefault()));
        addLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditAddLogicalFileText(Locale.getDefault()));
        addLogicalFileMenuItem.setEnabled(false);
        addLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(addLogicalFileMenuItem);

        updateLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.EDIT, IconSet.SIZE_16X16, Locale.getDefault()));
        updateLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditUpdateLogicalFileText(Locale.getDefault()));
        updateLogicalFileMenuItem.setEnabled(false);
        updateLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                updateLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(updateLogicalFileMenuItem);

        removeLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.REMOVE, IconSet.SIZE_16X16, Locale.getDefault()));
        removeLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditRemoveLogicalFileText(Locale.getDefault()));
        removeLogicalFileMenuItem.setEnabled(false);
        removeLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                removeLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(removeLogicalFileMenuItem);

        editMenu.add(jSeparator4);

        addTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.ADD, IconSet.SIZE_16X16, Locale.getDefault()));
        addTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditAddTransactionText(Locale.getDefault()));
        addTransactionMenuItem.setEnabled(false);
        addTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(addTransactionMenuItem);

        updateTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.EDIT, IconSet.SIZE_16X16, Locale.getDefault()));
        updateTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditUpdateTransactionText(Locale.getDefault()));
        updateTransactionMenuItem.setEnabled(false);
        updateTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                updateTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(updateTransactionMenuItem);

        removeTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.REMOVE, IconSet.SIZE_16X16, Locale.getDefault()));
        removeTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditRemoveTransactionText(Locale.getDefault()));
        removeTransactionMenuItem.setEnabled(false);
        removeTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                removeTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(removeTransactionMenuItem);

        menuBar.add(editMenu);

        toolsMenu.setText(PhysicalFileFrameBundle.getToolsMenuText(Locale.getDefault()));
        generatorMenuItem.setIcon(this.getIconSet().getIcon(IconSet.NEW, IconSet.SIZE_16X16, Locale.getDefault()));
        generatorMenuItem.setText(PhysicalFileFrameBundle.getToolsGeneratorText(Locale.getDefault()));
        generatorMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                generatorMenuItemActionPerformed(evt);
            }
        });

        toolsMenu.add(generatorMenuItem);

        bankDirectoryMenuItem.setAction(new SearchBankleitzahlAction(null, false));
        toolsMenu.add(bankDirectoryMenuItem);

        menuBar.add(toolsMenu);

        this.initLafMenu();
        lafMenu.setText(PhysicalFileFrameBundle.getLafMenuText(Locale.getDefault()));
        menuBar.add(lafMenu);

        setJMenuBar(menuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, logicalFilePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(logicalFilePanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowOpened(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowOpened
    {//GEN-HEADEREND:event_formWindowOpened
        this.setExtendedState(JFrame.MAXIMIZED_BOTH);
    }//GEN-LAST:event_formWindowOpened

    private void saveAsMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_saveAsMenuItemActionPerformed
    {//GEN-HEADEREND:event_saveAsMenuItemActionPerformed
        final JFileChooser chooser = new JFileChooser();
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setMultiSelectionEnabled(false);

        final Implementations formatters =
            ModelFactory.getModel().getModules().
            getSpecification(PhysicalFileFormatter.class.getName()).
            getImplementations();

        for(int i = formatters.size() - 1; i >= 0; i--)
        {
            final PhysicalFileFormatter formatter =
                (PhysicalFileFormatter) ContainerFactory.
                getContainer().getImplementation(
                PhysicalFileFormatter.class,
                formatters.getImplementation(i).getName());

            chooser.addChoosableFileFilter(
                new PhysicalFileFormatterFileFilter(formatter));

        }

        final int ret = chooser.
            showSaveDialog(getUserInterface().getParent());

        if(ret == JFileChooser.APPROVE_OPTION)
        {
            final PhysicalFileFormatter formatter =
                ((PhysicalFileFormatterFileFilter) chooser.
                getFileFilter()).getFormatter();

            final Thread t = new Thread()
            {
                public void run()
                {
                    try
                    {
                        setWorking(true);
                        formatter.format(pFile, chooser.getSelectedFile());
                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    finally
                    {
                        setWorking(false);
                    }
                }
            };

            t.setUncaughtExceptionHandler(this.getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_saveAsMenuItemActionPerformed

    private void closeMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeMenuItemActionPerformed
    {//GEN-HEADEREND:event_closeMenuItemActionPerformed
        this.setPhysicalFile(null);
    }//GEN-LAST:event_closeMenuItemActionPerformed

    private void generatorMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_generatorMenuItemActionPerformed
    {//GEN-HEADEREND:event_generatorMenuItemActionPerformed
        final GeneratorDialog dialog = this.getUserInterface().
            getGeneratorDialog();

        this.getUserInterface().push(dialog);
        dialog.setVisible(true);
        this.getUserInterface().pop();
    }//GEN-LAST:event_generatorMenuItemActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
    {//GEN-HEADEREND:event_formWindowClosing
        if(!this.isWorking())
        {
            this.setVisible(false);
            this.dispose();
        }
    }//GEN-LAST:event_formWindowClosing

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_exitMenuItemActionPerformed
    {//GEN-HEADEREND:event_exitMenuItemActionPerformed
        this.setVisible(false);
        this.dispose();
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void removeTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_removeTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_removeTransactionMenuItemActionPerformed
        final Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    setWorking(true);
                    lFile.removeTransaction(selectedTransaction.getIndex());
                    selectedTransaction = null;
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                finally
                {
                    setWorking(false);
                    updateLogicalFile();
                }
            }
        };

        t.setUncaughtExceptionHandler(this.getExceptionHandler());
        t.start();
    }//GEN-LAST:event_removeTransactionMenuItemActionPerformed

    private void removeLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_removeLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_removeLogicalFileMenuItemActionPerformed
        final Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    setWorking(true);
                    pFile.remove(getListModel().indexOf(lFile));
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                finally
                {
                    setWorking(false);
                    setLogicalFile(null);
                    updatePhysicalFile();
                }
            }
        };

        t.setUncaughtExceptionHandler(this.getExceptionHandler());
        t.start();
    }//GEN-LAST:event_removeLogicalFileMenuItemActionPerformed

    private void addTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_addTransactionMenuItemActionPerformed
        try
        {
            final Transaction t;
            final TransactionDialog dialog = getUserInterface().
                getTransactionDialog();

            dialog.setDebit(lFile.getHeader().getType().isDebitAllowed());
            dialog.setRemittance(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setDate(lFile.getHeader().getSchedule().getCreateDate());
            dialog.setTransaction(null);

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();

            t = dialog.getTransaction();
            if(t != null && dialog.getReturnStatus() ==
                TransactionDialog.RET_OK)
            {
                final Thread th = new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            setWorking(true);
                            lFile.createTransaction(t);
                        }
                        catch(IllegalArgumentException e)
                        {
                            onIllegalArgument(e);
                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        finally
                        {
                            setWorking(false);
                            updateLogicalFile();
                        }
                    }
                };

                th.setUncaughtExceptionHandler(getExceptionHandler());
                th.start();
            }
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_addTransactionMenuItemActionPerformed

    private void updateLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_updateLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_updateLogicalFileMenuItemActionPerformed
        try
        {
            final Header header;
            final HeaderDialog dialog = getUserInterface().getHeaderDialog();

            dialog.setDebitSupported(lFile.getHeader().getType().
                isDebitAllowed());

            dialog.setRemittanceSupported(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setHeader(lFile.getHeader());

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();

            header = dialog.getHeader();
            if(dialog.getReturnStatus() == HeaderDialog.RET_OK &&
                header != null && !header.equals(lFile.getHeader()))
            {
                final Thread t = new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            setWorking(true);
                            lFile.setHeader(header);
                        }
                        catch(IllegalArgumentException e)
                        {
                            onIllegalArgument(e);
                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        finally
                        {
                            setWorking(false);
                            updateLogicalFile();
                        }
                    }
                };

                t.setUncaughtExceptionHandler(getExceptionHandler());
                t.start();
            }
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_updateLogicalFileMenuItemActionPerformed

    private void addLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_addLogicalFileMenuItemActionPerformed
        final Header header;
        final HeaderDialog dialog = getUserInterface().getHeaderDialog();

        dialog.setDebitSupported(true);
        dialog.setRemittanceSupported(true);
        dialog.setHeader(null);

        getUserInterface().push(dialog);
        dialog.setVisible(true);
        getUserInterface().pop();

        header = dialog.getHeader();
        if(dialog.getReturnStatus() == HeaderDialog.RET_OK &&
            header != null)
        {
            final Thread t = new Thread()
            {
                public void run()
                {
                    LogicalFile logicalFile = null;

                    try
                    {
                        setWorking(true);
                        logicalFile = pFile.add(header);
                    }
                    catch(IllegalArgumentException e)
                    {
                        onIllegalArgument(e);
                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    finally
                    {
                        setWorking(false);
                        setLogicalFile(logicalFile);
                        updatePhysicalFile();
                    }
                }
            };

            t.setUncaughtExceptionHandler(getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_addLogicalFileMenuItemActionPerformed

    private void newMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newMenuItemActionPerformed
    {//GEN-HEADEREND:event_newMenuItemActionPerformed
        final int option;
        final JFileChooser chooser = new JFileChooser();

        getUserInterface().push(chooser);
        option = chooser.showSaveDialog(PhysicalFileFrame.this);
        getUserInterface().pop();

        if(option == JFileChooser.APPROVE_OPTION)
        {
            final File file = chooser.getSelectedFile();
            if(file != null)
            {
                final Thread t = new Thread()
                {
                    public void run()
                    {
                        PhysicalFile physFile = null;

                        try
                        {
                            setWorking(true);

                            final FileOperations ops =
                                new RandomAccessFileOperations(
                                new RandomAccessFile(file, "rw"));

                            ops.setLength(0L);
                            physFile = getPhysicalFileFactory().
                                getPhysicalFile(ops);

                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        catch(PhysicalFileException e)
                        {
                            final Object[] messages =
                                new Object[e.getMessages().length];

                            for(int i = 0; i < messages.length && i < 25; i++)
                            {
                                messages[i] = e.getMessages()[i].
                                    getText(Locale.getDefault());

                            }

                            onError(e, messages);
                        }
                        finally
                        {
                            setWorking(false);
                            setPhysicalFile(physFile);
                        }
                    }
                };

                t.setUncaughtExceptionHandler(getExceptionHandler());
                t.start();
            }
        }
    }//GEN-LAST:event_newMenuItemActionPerformed

    private void updateTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_updateTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_updateTransactionMenuItemActionPerformed
        try
        {
            final TransactionDialog dialog = getUserInterface().
                getTransactionDialog();

            dialog.setDebit(lFile.getHeader().getType().
                isDebitAllowed());

            dialog.setRemittance(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setDate(lFile.getHeader().getSchedule().getCreateDate());
            dialog.setTransaction(this.selectedTransaction);

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();

            final Transaction updated = dialog.getTransaction();

            if(dialog.getReturnStatus() == TransactionDialog.RET_OK &&
                updated != null && !updated.equals(selectedTransaction))
            {
                final Thread t = new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            setWorking(true);

                            final int index = selectedTransaction.getIndex();

                            lFile.setTransaction(index, updated);
                            selectedTransaction = new TransactionView(index,
                                lFile.getTransaction(index));

                        }
                        catch(IllegalArgumentException e)
                        {
                            onIllegalArgument(e);
                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        finally
                        {
                            setWorking(false);
                            updateLogicalFile();
                        }
                    }
                };

                t.setUncaughtExceptionHandler(getExceptionHandler());
                t.start();
            }
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_updateTransactionMenuItemActionPerformed

    private void logicalFileListValueChanged(javax.swing.event.ListSelectionEvent evt)//GEN-FIRST:event_logicalFileListValueChanged
    {//GEN-HEADEREND:event_logicalFileListValueChanged
        if(!evt.getValueIsAdjusting())
        {
            this.setLogicalFile((LogicalFile) this.logicalFileList.
                getSelectedValue());

        }
    }//GEN-LAST:event_logicalFileListValueChanged

    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_openMenuItemActionPerformed
    {//GEN-HEADEREND:event_openMenuItemActionPerformed
        final File file;
        final int option;
        final JFileChooser chooser = new JFileChooser();

        getUserInterface().push(chooser);
        option = chooser.showOpenDialog(PhysicalFileFrame.this);
        getUserInterface().pop();

        file = chooser.getSelectedFile();
        if(file != null && option == JFileChooser.APPROVE_OPTION)
        {
            final Thread t = new Thread()
            {
                public void run()
                {
                    PhysicalFile physFile = null;

                    try
                    {
                        setWorking(true);
                        physFile = getPhysicalFileFactory().getPhysicalFile(
                            new RandomAccessFileOperations(
                            new RandomAccessFile(file, "rw")));

                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    catch(PhysicalFileException e)
                    {
                        final Object[] messages =
                            new Object[e.getMessages().length];

                        for(int i = 0; i < messages.length && i < 25; i++)
                        {
                            messages[i] = e.getMessages()[i].
                                getText(Locale.getDefault());

                        }

                        onError(e, messages);
                    }
                    finally
                    {
                        setWorking(false);
                        setPhysicalFile(physFile);
                    }
                }
            };

            t.setUncaughtExceptionHandler(getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_openMenuItemActionPerformed

    private void transactionTableValueChanged(final ListSelectionEvent evt)
    {
        if(!evt.getValueIsAdjusting())
        {
            if(evt.getFirstIndex() < 0)
            {
                this.selectedTransaction = null;
                this.updateTransactionMenuItem.setEnabled(false);
                this.removeTransactionMenuItem.setEnabled(false);
            }
            else
            {
                this.selectedTransaction = getTableModel().getEntities().
                    get(transactionTable.getSelectedRow());

                this.updateTransactionMenuItem.setEnabled(true);
                this.removeTransactionMenuItem.setEnabled(true);
            }
        }
    }

    private void onError(final Throwable t, final Object messages)
    {
        UIManager.getLookAndFeel().provideErrorFeedback(
            getUserInterface().getParent());

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final String name = t.getClass().getName();
                JOptionPane.showMessageDialog(getUserInterface().getParent(),
                    messages, name.substring(name.lastIndexOf('.') + 1),
                    JOptionPane.ERROR_MESSAGE);

            }
        });
    }

    private void onIllegalArgument(final IllegalArgumentException e)
    {
        UIManager.getLookAndFeel().provideErrorFeedback(
            getUserInterface().getParent());

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                JOptionPane.showMessageDialog(getUserInterface().getParent(),
                    PhysicalFileFrameBundle.
                    getIllegalArgumentMessage(Locale.getDefault()).
                    format(new Object[] { e.getMessage() }),
                    PhysicalFileFrameBundle.
                    getIllegalArgumentTitleText(Locale.getDefault()),
                    JOptionPane.ERROR_MESSAGE);

            }
        });
    }

    private void updatePhysicalFile()
    {
        final Thread t = new Thread()
        {
            public void run()
            {
                final LogicalFile prevSelection = lFile;

                try
                {
                    setWorking(true);

                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            addLogicalFileMenuItem.setEnabled(pFile != null);
                            closeMenuItem.setEnabled(pFile != null);
                            saveAsMenuItem.setEnabled(pFile != null);
                            getTableModel().clear();
                            getListModel().clear();
                        }
                    });

                    if(pFile != null)
                    {
                        final LogicalFile lFile;
                        final Collection<LogicalFile> files =
                            new ArrayList<LogicalFile>(pFile.count());

                        for(int i = 0; i < pFile.count(); i++)
                        {
                            files.add(pFile.get(i));
                        }

                        SwingUtilities.invokeLater(new Runnable()
                        {
                            public void run()
                            {
                                getListModel().addAll(files);
                            }
                        });
                    }
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                finally
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            logicalFileList.setSelectedValue(
                                prevSelection, true);

                        }
                    });

                    setWorking(false);
                }
            }
        };

        t.setUncaughtExceptionHandler(getExceptionHandler());
        t.start();
    }

    private void updateLogicalFile()
    {
        final Thread t = new Thread()
        {
            public void run()
            {
                final TransactionView prevSelection = selectedTransaction;

                try
                {
                    setWorking(true);

                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            getTableModel().clear();
                        }
                    });

                    if(lFile != null)
                    {
                        final Collection<TransactionView> col =
                            new ArrayList(lFile.getChecksum().
                            getTransactionCount());

                        final int count = lFile.getChecksum().
                            getTransactionCount();

                        for(int i = 0; i < count; i++)
                        {
                            col.add(new TransactionView(i,
                                lFile.getTransaction(i)));

                        }

                        SwingUtilities.invokeLater(new Runnable()
                        {
                            public void run()
                            {
                                getTableModel().addAll(col);
                            }
                        });
                    }
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                finally
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            updateLogicalFileMenuItem.setEnabled(lFile != null);
                            removeLogicalFileMenuItem.setEnabled(lFile != null);
                            addTransactionMenuItem.setEnabled(lFile != null);

                            if(prevSelection != null)
                            {
                                final int index = getTableModel().getEntities().
                                    indexOf(prevSelection);

                                transactionTable.getSelectionModel().
                                    setSelectionInterval(index, index);

                                transactionTable.scrollRectToVisible(
                                    transactionTable.getCellRect(
                                    index, 0, true));

                            }
                            else
                            {
                                transactionTable.getSelectionModel().
                                    clearSelection();

                            }
                        }
                    });

                    setWorking(false);
                }
            }
        };

        t.setUncaughtExceptionHandler(getExceptionHandler());
        t.start();
    }

    private void initLafMenu()
    {
        final ButtonGroup buttonGroup = new ButtonGroup();
        final UIManager.LookAndFeelInfo[] info =
            UIManager.getInstalledLookAndFeels();

        Arrays.sort(info, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof UIManager.LookAndFeelInfo;
                assert o2 instanceof UIManager.LookAndFeelInfo;

                return ((UIManager.LookAndFeelInfo) o1).getName().
                    compareTo(((UIManager.LookAndFeelInfo) o2).getName());

            }
        });

        for(int i = 0; i < info.length; i++)
        {
            final String lafClass = info[i].getClassName();
            final JRadioButtonMenuItem item = new JRadioButtonMenuItem();

            buttonGroup.add(item);

            item.setText(info[i].getName());
            if(UIManager.getSystemLookAndFeelClassName().
                equals(info[i].getClassName()))
            {
                item.setSelected(true);
            }

            item.addActionListener(new ActionListener()
            {
                public void actionPerformed(final ActionEvent e)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            try
                            {
                                UIManager.setLookAndFeel(lafClass);
                                SwingUtilities.updateComponentTreeUI(
                                    PhysicalFileFrame.this);

                                pack();

                                SwingUtilities.updateComponentTreeUI(
                                    getUserInterface().
                                    getBankleitzahlenVerzeichnisDialog());

                                getUserInterface().
                                    getBankleitzahlenVerzeichnisDialog().pack();

                                SwingUtilities.updateComponentTreeUI(
                                    getUserInterface().getHeaderDialog());

                                getUserInterface().getHeaderDialog().pack();

                                SwingUtilities.updateComponentTreeUI(
                                    getUserInterface().getTransactionDialog());

                                getUserInterface().getTransactionDialog().
                                    pack();

                                SwingUtilities.updateComponentTreeUI(
                                    getUserInterface().getGeneratorDialog());

                                getUserInterface().getGeneratorDialog().pack();
                            }
                            catch(ClassNotFoundException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(InstantiationException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(IllegalAccessException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(UnsupportedLookAndFeelException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                        }
                    });
                }
            });

            this.lafMenu.add(item);
        }
    }

    private PhysicalFileFrameTableModel getTableModel()
    {
        return (PhysicalFileFrameTableModel) this.transactionTable.getModel();
    }

    private PhysicalFileFrameListModel getListModel()
    {
        return (PhysicalFileFrameListModel) this.logicalFileList.getModel();
    }

    private Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        return this.getApplication().getExceptionHandler();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem addLogicalFileMenuItem;
    private javax.swing.JMenuItem addTransactionMenuItem;
    private javax.swing.JMenuItem bankDirectoryMenuItem;
    private javax.swing.JMenuItem closeMenuItem;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenuItem generatorMenuItem;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JMenu lafMenu;
    private javax.swing.JList logicalFileList;
    private javax.swing.JPanel logicalFilePanel;
    private javax.swing.JScrollPane logicalFileScrollPane;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem newMenuItem;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JMenuItem removeLogicalFileMenuItem;
    private javax.swing.JMenuItem removeTransactionMenuItem;
    private javax.swing.JMenuItem saveAsMenuItem;
    private javax.swing.JMenu toolsMenu;
    private javax.swing.JPanel transactionPanel;
    private javax.swing.JScrollPane transactionScrollPane;
    private javax.swing.JTable transactionTable;
    private javax.swing.JMenuItem updateLogicalFileMenuItem;
    private javax.swing.JMenuItem updateTransactionMenuItem;
    // End of variables declaration//GEN-END:variables

    /** Physical file currently being edited or {@code null}. */
    private transient PhysicalFile pFile;

    /** Logical file currently selected or {@code null}. */
    private transient LogicalFile lFile;

    /** Currently selected transaction or {@code null}. */
    private transient TransactionView selectedTransaction;

    /** Flag indicating if an asynchronous operation is in progress. */
    private transient boolean working;

    //-------------------------------------------------------PhysicalFileFrame--

}

/**
 * Model for the table holding {@code TransactionView} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
class PhysicalFileFrameTableModel implements TableModel
{
    //--TableModel--------------------------------------------------------------

    /**
     * {@code TableModelEvent} listener list.
     * @serial
     */
    private EventListenerList tableListeners = new EventListenerList();

    public int getRowCount()
    {
        return this.getEntities().size();
    }

    public int getColumnCount()
    {
        return 7;
    }

    public String getColumnName(int columnIndex)
    {
        switch(columnIndex)
        {
            case 0: return PhysicalFileFrameBundle.
                getTransactionExecutiveNameText(Locale.getDefault());

            case 1: return PhysicalFileFrameBundle.
                getTransactionExecutiveAccountText(Locale.getDefault());

            case 2: return PhysicalFileFrameBundle.
                getTransactionExecutiveBankText(Locale.getDefault());

            case 3: return PhysicalFileFrameBundle.
                getTransactionTargetNameText(Locale.getDefault());

            case 4: return PhysicalFileFrameBundle.
                getTransactionTargetAccountText(Locale.getDefault());

            case 5: return PhysicalFileFrameBundle.
                getTransactionTargetBankText(Locale.getDefault());

            case 6: return PhysicalFileFrameBundle.
                getTransactionReferenceText(Locale.getDefault());

            default: return null;

        }
    }

    public Class<?> getColumnClass(int columnIndex)
    {
        return String.class;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return false;
    }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
        if(rowIndex >= this.getEntities().size())
        {
            throw new IllegalArgumentException(Integer.toString(rowIndex));
        }
        if(columnIndex >= this.getColumnCount())
        {
            throw new IllegalArgumentException(Integer.toString(columnIndex));
        }

        final Transaction t = this.getEntities().get(rowIndex);
        switch(columnIndex)
        {
            case 0: return t.getExecutiveName() == null ?
                "" : t.getExecutiveName().format();

            case 1: return t.getExecutiveAccount() == null ?
                "" : t.getExecutiveAccount().format(
                Kontonummer.ELECTRONIC_FORMAT);

            case 2: return t.getExecutiveBank() == null ?
                "" : t.getExecutiveBank().format(
                Bankleitzahl.ELECTRONIC_FORMAT);

            case 3: return t.getTargetName() == null ?
                "" : t.getTargetName().format();

            case 4: return t.getTargetAccount() == null ?
                "" : t.getTargetAccount().format(
                Kontonummer.ELECTRONIC_FORMAT);

            case 5: return t.getTargetBank() == null ?
                "" : t.getTargetBank().format(
                Bankleitzahl.ELECTRONIC_FORMAT);

            case 6: return t.getReference() == null ? ""
                : t.getReference().format(Referenznummer11.ELECTRONIC_FORMAT);

            default: return t.toString();
        }
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
    }

    public final void addTableModelListener(final TableModelListener l)
    {
        this.tableListeners.add(TableModelListener.class, l);
    }

    public final void removeTableModelListener(final TableModelListener l)
    {
        this.tableListeners.remove(TableModelListener.class, l);
    }

    //--------------------------------------------------------------TableModel--
    //--TableModel--------------------------------------------------------------

    /**
     * Collection of {@code Transaction} instances.
     * @serial
     */
    private List<TransactionView> entities;

    /** Sorts {@code Transaction} instances. */
    private static Comparator TRANSACTION_SORTER = new Comparator()
    {
        public int compare(final Object o1, final Object o2)
        {
            if(!(o1 instanceof Transaction))
            {
                throw new IllegalArgumentException(o1.getClass().getName());
            }
            if(!(o2 instanceof Transaction))
            {
                throw new IllegalArgumentException(o2.getClass().getName());
            }

            int ret;
            final Transaction t1 = (Transaction) o1;
            final Transaction t2 = (Transaction) o2;

            if(t1.getExecutiveName() == null)
            {
                ret = t2.getExecutiveName() == null ? 0 : -1;
            }
            else
            {
                ret = t1.getExecutiveName().compareTo(t2.getExecutiveName());
            }

            if(ret == 0)
            {
                if(t1.getExecutiveAccount() == null)
                {
                    ret = t2.getExecutiveAccount() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getExecutiveAccount().compareTo(
                        t2.getExecutiveAccount());

                }
            }

            if(ret == 0)
            {
                if(t1.getExecutiveBank() == null)
                {
                    ret = t2.getExecutiveBank() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getExecutiveBank().compareTo(
                        t2.getExecutiveBank());

                }
            }

            if(ret == 0)
            {
                if(t1.getTargetName() == null)
                {
                    ret = t2.getTargetName() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetName().compareTo(t2.getTargetName());
                }
            }

            if(ret == 0)
            {
                if(t1.getTargetAccount() == null)
                {
                    ret = t2.getTargetAccount() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetAccount().compareTo(
                        t2.getTargetAccount());

                }
            }

            if(ret == 0)
            {
                if(t1.getTargetBank() == null)
                {
                    ret = t2.getTargetBank() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetBank().compareTo(t2.getTargetBank());
                }
            }

            if(ret == 0)
            {
                if(t1.getReference() == null)
                {
                    ret = t2.getReference() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getReference().compareTo(t2.getReference());
                }
            }

            return ret;
        }
    };

    /**
     * Gets the entities of the model.
     *
     * @return all entities of the model.
     */
    public List<TransactionView> getEntities()
    {
        if(this.entities == null)
        {
            this.entities = Collections.synchronizedList(new ArrayList(50));
        }

        return this.entities;
    }

    protected void add(final TransactionView transaction)
    {
        if(transaction == null)
        {
            throw new NullPointerException("transaction");
        }

        this.getEntities().add(transaction);
        Collections.sort(this.getEntities(), TRANSACTION_SORTER);
        this.fireTableChanged(new TableModelEvent(this));
    }

    protected void addAll(final Collection<TransactionView> transactions)
    {
        if(transactions != null)
        {
            this.getEntities().addAll(transactions);
        }
        Collections.sort(this.getEntities(), TRANSACTION_SORTER);
        this.fireTableChanged(new TableModelEvent(this));
    }

    protected void remove(final Transaction transaction)
    {
        if(transaction == null)
        {
            throw new NullPointerException("transaction");
        }

        final int row = this.getEntities().indexOf(transaction);
        if(row != -1)
        {
            this.getEntities().remove(transaction);
            this.fireTableChanged(new TableModelEvent(this, row, row,
                TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));

        }
    }

    protected void clear()
    {
        this.getEntities().clear();
        this.fireTableChanged(new TableModelEvent(this));
    }


    /**
     * Notifies all registered {@code TableModelListener}s about {@code event}.
     *
     * @param event the event for the listeners.
     */
    protected void fireTableChanged(final TableModelEvent event)
    {
        if(event != null)
        {
            final Object[] listeners = this.tableListeners.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2)
            {
                if (listeners[i] == TableModelListener.class)
                {
                    ((TableModelListener) listeners[i + 1]).tableChanged(event);
                }
            }
        }
    }

    //--------------------------------------------------------------TableModel--

}

/**
 * Model for the list holding {@code LogicalFile} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
class PhysicalFileFrameListModel extends AbstractListModel
{
    //--AbstractListModel-------------------------------------------------------

    public final int getSize()
    {
        return this.getEntities().size();
    }

    public final Object getElementAt(int index)
    {
        return this.getEntities().get(index);
    }

    //-------------------------------------------------------AbstractListModel--
    //--PhysicalFileFrameListModel----------------------------------------------

    /**
     * Collection of {@code LogicalFile} instances.
     * @serial
     */
    private List<LogicalFile> entities;

    /**
     * Gets the entities of the model.
     *
     * @return all entities of the model.
     */
    public List<LogicalFile> getEntities()
    {
        if(this.entities == null)
        {
            this.entities = Collections.synchronizedList(new ArrayList(50));
        }

        return this.entities;
    }

    protected void clear()
    {
        final int oldSize = this.getSize();
        this.getEntities().clear();
        this.fireIntervalRemoved(this, 0, oldSize);
    }

    protected void addAll(final Collection<LogicalFile> files)
    {
        final int oldSize = this.getSize();
        this.getEntities().addAll(files);
        this.fireIntervalAdded(this, oldSize, this.getSize());
    }

    protected void remove(final int index)
    {
        this.getEntities().remove(index);
        this.fireIntervalRemoved(this, index, index);
    }

    protected int indexOf(final LogicalFile lFile)
    {
        return this.getEntities().indexOf(lFile);
    }

    //----------------------------------------------PhysicalFileFrameListModel--

}

/**
 * Extension to {@code Transaction} adding a property holding the index of the
 * transaction in a logical file.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
class TransactionView extends Transaction
{
    //--TransactionView---------------------------------------------------------

    /**
     * Index of the transaction in the logical file.
     * @serial
     */
    private final int index;

    /**
     * Creates a new {@code TransactionView} instance.
     *
     * @param index the index of {@code transaction} in a logical file.
     * @param transaction the data to initialize the new instance with.
     */
    public TransactionView(final int index, final Transaction transaction)
    {
        super();
        this.index = index;
        this.setAmount(transaction.getAmount());
        this.setCurrency(transaction.getCurrency());
        this.setDescription(transaction.getDescription());
        this.setExecutiveAccount(transaction.getExecutiveAccount());
        this.setExecutiveBank(transaction.getExecutiveBank());
        this.setExecutiveExt(transaction.getExecutiveExt());
        this.setExecutiveName(transaction.getExecutiveName());
        this.setPrimaryBank(transaction.getPrimaryBank());
        this.setReference(transaction.getReference());
        this.setTargetAccount(transaction.getTargetAccount());
        this.setTargetBank(transaction.getTargetBank());
        this.setTargetExt(transaction.getTargetExt());
        this.setTargetName(transaction.getTargetName());
        this.setType(transaction.getType());
    }

    /**
     * Gets the index of the transaction in the logical file.
     *
     * @return the index of the transaction in the logical file.
     */
    public int getIndex()
    {
        return this.index;
    }

    //---------------------------------------------------------TransactionView--
}

/**
 * Adapts {@code PhysicalFileFormatter} instances to {@code FileFilter}s.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
class PhysicalFileFormatterFileFilter extends FileFilter
{
    //--PhysicalFileFormatterFileFilter-----------------------------------------

    /** {@code PhysicalFileFormatter} instance adapted to {@code FileFilter}. */
    private PhysicalFileFormatter formatter;

    /**
     * Creates a new {@code PhysicalFileFormatterFileFilter} instance taking
     * the {@code PhysicalFileFormatter} to adapt.
     *
     * @param formatter the {@code PhysicalFileFormatter} to adapt.
     *
     * @throws NullPointerException if {@code formatter} is {@code null}.
     */
    PhysicalFileFormatterFileFilter(final PhysicalFileFormatter formatter)
    {
        if(formatter == null)
        {
            throw new NullPointerException("formatter");
        }

        this.formatter = formatter;
    }

    /**
     * Gets the {@code PhysicalFileFormatter} instance backing the instance.
     *
     * @return the {@code PhysicalFileFormatter} instance backing the instance.
     */
    PhysicalFileFormatter getFormatter()
    {
        return this.formatter;
    }

    //-----------------------------------------PhysicalFileFormatterFileFilter--
    //--FileFilter--------------------------------------------------------------

    public boolean accept(final File f)
    {
        return this.formatter.accept(f);
    }

    public String getDescription()
    {
        return this.formatter.getDescription(Locale.getDefault());
    }

    //--------------------------------------------------------------FileFilter--
}
