/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.swing;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.lang.reflect.InvocationTargetException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Locale;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlInfo;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.dtaus.swing.Editor;
import org.jdtaus.banking.dtaus.swing.EditorGUI;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ImplementationException;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;

/**
 * Dialog to display and search the {@code BankleitzahlenVerzeichnis}.
 * <p>The dialog supports selecting a single {@code BankleitzahlInfo} instance
 * which can be obtained via property {@code selectedBank}.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: BankleitzahlenVerzeichnisDialog.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
public class BankleitzahlenVerzeichnisDialog extends JDialog
    implements ContainerInitializer
{

    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(BankleitzahlenVerzeichnisDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

        this.doSearch();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
           ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(BankleitzahlenVerzeichnisDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(BankleitzahlenVerzeichnisDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency1;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(BankleitzahlenVerzeichnisDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(BankleitzahlenVerzeichnisDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency0;

    /**
     * Gets the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     *
     * @return the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(BankleitzahlenVerzeichnisDialog.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(BankleitzahlenVerzeichnisDialog.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //--------------------------------------------------------------Properties--
    //--BankleitzahlenVerzeichnisDialog-----------------------------------------

    /** A return status code - returned if Cancel button has been pressed. */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed. */
    public static final int RET_OK = 1;

    /** Mutex for the search thread. */
    private final Object searchMutex = new Object();

    /**
     * Creates a new {@code BankleitzahlenVerzeichnisDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public BankleitzahlenVerzeichnisDialog(final Frame owner,
        final boolean modal)
    {
        super(owner, modal);
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the selected bank.
     *
     * @return the selected bank or {@code null} for no selection.
     */
    public BankleitzahlInfo getSelectedBank()
    {
        return this.selectedBank;
    }

    /**
     * Sets the selected bank.
     *
     * @param bank the bank to select or {@code null} for no selection.
     */
    public void setSelectedBank(final BankleitzahlInfo bank)
    {
        final Thread t = new Thread()
        {
            public void run()
            {
                synchronized(searchMutex)
                {
                    selectedBank = bank;
                    updateSelectedBank();
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            if(bank != null)
                            {
                                final int index = getModel().getEntities().
                                    indexOf(bank);

                                directoryTable.getSelectionModel().
                                    setSelectionInterval(index, index);

                            }
                            else
                            {
                                directoryTable.getSelectionModel().
                                    clearSelection();

                            }

                            directoryTable.scrollRectToVisible(
                                directoryTable.getCellRect(directoryTable.
                                getSelectedRow(), 0, true));

                        }
                    });
                }
            }
        };

        t.setUncaughtExceptionHandler(this.getExceptionHandler());
        t.start();
    }

    /**
     * Flag indicating if an asynchronous operation is in progress.
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    public boolean isWorking()
    {
        return this.working;
    }

    /**
     * Setter for property {@code working}.
     * <p>This method will disable the dialog's components when {@code working}
     * is {@code true} and should be used during asynchronous operations to
     * disable the dialog without closing it.</p>
     *
     * @return {@code true} if an asynchronous operation is in progress;
     * {@code false} if not.
     */
    protected void setWorking(final boolean working)
    {
        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {
                public void run()
                {
                    setCursor(Cursor.getPredefinedCursor(working ?
                        Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

                    bankNameTextField.setEnabled(!working);
                    //bankleitzahlTextField.setEnabled(!working);
                    branchOfficeCheckBox.setEnabled(!working);
                    closeButton.setEnabled(!working);
                    cityTextField.setEnabled(!working);
                    directoryTable.setEnabled(!working);
                    okButton.setEnabled(!working);
                    postalCodeTextField.setEnabled(!working);
                    searchButton.setEnabled(!working);
                    BankleitzahlenVerzeichnisDialog.this.working = working;
                }
            });
        }
        catch(InterruptedException e)
        {
            throw new ImplementationException(
                BankleitzahlenVerzeichnisDialog.META, e);
        }
        catch(InvocationTargetException e)
        {
            throw new ImplementationException(
                BankleitzahlenVerzeichnisDialog.META,
                e.getTargetException() == null ? e : e.getTargetException());

        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        searchPanel = new javax.swing.JPanel();
        bankleitzahlLabel = new javax.swing.JLabel();
        bankleitzahlTextField = new javax.swing.JTextField();
        postalCodeLabel = new javax.swing.JLabel();
        postalCodeTextField = new javax.swing.JTextField();
        bankNameLabel = new javax.swing.JLabel();
        bankNameTextField = new javax.swing.JTextField();
        cityLabel = new javax.swing.JLabel();
        cityTextField = new javax.swing.JTextField();
        searchButton = new javax.swing.JButton();
        branchOfficeCheckBox = new javax.swing.JCheckBox();
        directoryPanel = new javax.swing.JPanel();
        directoryScrollPane = new javax.swing.JScrollPane();
        directoryTable = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(BankleitzahlenVerzeichnisDialogBundle.getTitleText(Locale.getDefault()));
        setName("Bankcode search");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText(BankleitzahlenVerzeichnisDialogBundle.getOkButtonText(Locale.getDefault()));
        okButton.setEnabled(false);
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        closeButton.setText(BankleitzahlenVerzeichnisDialogBundle.getCloseButtonText(Locale.getDefault()));
        closeButton.setDefaultCapable(false);
        closeButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeButtonActionPerformed(evt);
            }
        });

        searchPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(BankleitzahlenVerzeichnisDialogBundle.getSearchPanelTitleText(Locale.getDefault())));
        bankleitzahlLabel.setText(BankleitzahlenVerzeichnisDialogBundle.getBankleitzahlLabelText(Locale.getDefault()));

        bankleitzahlTextField.setEnabled(false);

        postalCodeLabel.setText(BankleitzahlenVerzeichnisDialogBundle.getPostleitzahlLabelText(Locale.getDefault()));

        bankNameLabel.setText(BankleitzahlenVerzeichnisDialogBundle.getBanknameLabelText(Locale.getDefault()));

        cityLabel.setText(BankleitzahlenVerzeichnisDialogBundle.getCityLabelText(Locale.getDefault()));

        searchButton.setText(BankleitzahlenVerzeichnisDialogBundle.getSearchButtonText(Locale.getDefault()));
        searchButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                searchButtonActionPerformed(evt);
            }
        });

        branchOfficeCheckBox.setText(BankleitzahlenVerzeichnisDialogBundle.getBranchOfficesCheckBoxText(Locale.getDefault()));
        branchOfficeCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        branchOfficeCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        branchOfficeCheckBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                branchOfficeCheckBoxActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout searchPanelLayout = new org.jdesktop.layout.GroupLayout(searchPanel);
        searchPanel.setLayout(searchPanelLayout);
        searchPanelLayout.setHorizontalGroup(
            searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(searchPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(searchPanelLayout.createSequentialGroup()
                        .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, bankNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, bankleitzahlLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 104, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(bankNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 220, Short.MAX_VALUE)
                            .add(bankleitzahlTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 220, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(cityLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(postalCodeLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 93, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(cityTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 230, Short.MAX_VALUE)
                            .add(postalCodeTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 230, Short.MAX_VALUE)))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, searchPanelLayout.createSequentialGroup()
                        .add(branchOfficeCheckBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 323, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 242, Short.MAX_VALUE)
                        .add(searchButton)))
                .addContainerGap())
        );
        searchPanelLayout.setVerticalGroup(
            searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(searchPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(postalCodeTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(bankleitzahlLabel)
                    .add(bankleitzahlTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(postalCodeLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bankNameLabel)
                    .add(bankNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(cityLabel)
                    .add(cityTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 17, Short.MAX_VALUE)
                .add(searchPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(searchButton)
                    .add(branchOfficeCheckBox))
                .addContainerGap())
        );

        directoryPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(BankleitzahlenVerzeichnisDialogBundle.getVerzeichnisPanelTitleText(Locale.getDefault())));
        directoryScrollPane.setAutoscrolls(true);
        directoryTable.setModel(new BankleitzahlenVerzeichnisDialogModel());
        directoryTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_ALL_COLUMNS);
        directoryTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        directoryTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            public void valueChanged(final ListSelectionEvent event)
            {
                directoryTableValueChanged(event);
            }
        });

        directoryScrollPane.setViewportView(directoryTable);

        org.jdesktop.layout.GroupLayout directoryPanelLayout = new org.jdesktop.layout.GroupLayout(directoryPanel);
        directoryPanel.setLayout(directoryPanelLayout);
        directoryPanelLayout.setHorizontalGroup(
            directoryPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(directoryScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 707, Short.MAX_VALUE)
        );
        directoryPanelLayout.setVerticalGroup(
            directoryPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(directoryScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 407, Short.MAX_VALUE)
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, directoryPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(closeButton))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, searchPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {closeButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(searchPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(directoryPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(closeButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void branchOfficeCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_branchOfficeCheckBoxActionPerformed
    {//GEN-HEADEREND:event_branchOfficeCheckBoxActionPerformed
        this.doSearch();
    }//GEN-LAST:event_branchOfficeCheckBoxActionPerformed

    private void searchButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchButtonActionPerformed
        this.doSearch();
    }//GEN-LAST:event_searchButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        this.doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeButtonActionPerformed
    {//GEN-HEADEREND:event_closeButtonActionPerformed
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_closeButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        if(!this.isWorking())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void doSearch()
    {
        //final String blz = this.bankleitzahlTextField.getText();
        final String name = this.bankNameTextField.getText();
        final String plz = this.postalCodeTextField.getText();
        final String city = this.cityTextField.getText();
        final BankleitzahlInfo prevSelection = this.selectedBank;

        final Thread t = new Thread()
        {
            public void run()
            {
                try
                {
                    setWorking(true);
                    synchronized(searchMutex)
                    {
                        getModel().clear();
                        getModel().addAll(Arrays.asList(
                            getBankleitzahlenVerzeichnis().search(
                            name != null && name.trim().length() > 0 ?
                                name.trim() : null,
                            plz != null && plz.trim().length() > 0 ?
                                plz.trim() : null,
                            city != null && city.trim().length() > 0 ?
                                city.trim() : null,
                            branchOfficeCheckBox.isSelected())));

                    }
                }
                catch(final IllegalArgumentException e)
                {
                    UIManager.getLookAndFeel().provideErrorFeedback(
                        getUserInterface().getParent());

                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            final String name = e.getClass().getName();
                            JOptionPane.showMessageDialog(
                                BankleitzahlenVerzeichnisDialog.this,
                                e.getMessage(),
                                name.substring(name.lastIndexOf('.') + 1),
                                JOptionPane.ERROR_MESSAGE);

                        }
                    });
                }
                finally
                {
                    setWorking(false);
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            if(prevSelection != null)
                            {
                                final int index = getModel().getEntities().
                                    indexOf(prevSelection);

                                if(index < 0)
                                {
                                    directoryTable.getSelectionModel().
                                        clearSelection();

                                }
                                else
                                {
                                    directoryTable.getSelectionModel().
                                        setSelectionInterval(index, index);

                                }

                                directoryTable.scrollRectToVisible(
                                    directoryTable.getCellRect(
                                    directoryTable.getSelectedRow(), 0,
                                    true));

                            }
                        }
                    });
                }
            }
        };

        t.setUncaughtExceptionHandler(this.getExceptionHandler());
        t.start();
    }

    /**
     * Gets the model of the dialog.
     *
     * @return the model of the dialog.
     */
    private BankleitzahlenVerzeichnisDialogModel getModel()
    {
        return (BankleitzahlenVerzeichnisDialogModel) directoryTable.getModel();
    }

    /**
     * Updates the view for a given selection.
     */
    private void updateSelectedBank()
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                if(selectedBank != null)
                {
                    final BankleitzahlInfo headOffice;
                    final NumberFormat plzFormat = new DecimalFormat("#####");
                    final StringBuffer tooltip = new StringBuffer(1000);

                    tooltip.append("<html>").append(selectedBank.getName());
                    tooltip.append("<br>").append(plzFormat.
                        format(selectedBank.getPostalCode())).append(" ").
                        append(selectedBank.getCity());

                    if(!selectedBank.isHeadOffice())
                    {
                        headOffice = getBankleitzahlenVerzeichnis().
                            getHeadOffice(selectedBank.getBankCode());

                        tooltip.append("<br><br><b>").append(
                            BankleitzahlenVerzeichnisDialogBundle.
                            getHeadOfficeLabelText(Locale.getDefault())).
                            append(" ");

                        headOffice.getBankCode().
                            format(Bankleitzahl.LETTER_FORMAT, tooltip);

                        tooltip.append("</b><br>").
                            append(headOffice.getName()).
                            append("<br>").append(plzFormat.format(
                            headOffice.getPostalCode())).append(" ").
                            append(headOffice.getCity());

                    }

                    tooltip.append("</html>");
                    directoryTable.setToolTipText(tooltip.toString());
                    bankleitzahlTextField.setText(selectedBank.getBankCode().
                        format(Bankleitzahl.LETTER_FORMAT));

                    okButton.setEnabled(true);
                }
                else
                {
                    directoryTable.setToolTipText(null);
                    bankleitzahlTextField.setText(null);
                    okButton.setEnabled(false);
                }
            }
        });
    }

    private void directoryTableValueChanged(final ListSelectionEvent event)
    {
        if(!event.getValueIsAdjusting())
        {
            if(directoryTable.getSelectedRow() < 0)
            {
                this.selectedBank = null;
            }
            else
            {
                this.selectedBank = this.getModel().getEntities().
                    get(directoryTable.getSelectedRow());

            }

            this.updateSelectedBank();
        }
    }

    private Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        return this.getApplication().getExceptionHandler();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel bankNameLabel;
    private javax.swing.JTextField bankNameTextField;
    private javax.swing.JLabel bankleitzahlLabel;
    private javax.swing.JTextField bankleitzahlTextField;
    private javax.swing.JCheckBox branchOfficeCheckBox;
    private javax.swing.JLabel cityLabel;
    private javax.swing.JTextField cityTextField;
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel directoryPanel;
    private javax.swing.JScrollPane directoryScrollPane;
    private javax.swing.JTable directoryTable;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel postalCodeLabel;
    private javax.swing.JTextField postalCodeTextField;
    private javax.swing.JButton searchButton;
    private javax.swing.JPanel searchPanel;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;
    private BankleitzahlInfo selectedBank;

    /**
     * Flag indicating if an asynchronous operation is in progress.
     * @serial
     */
    private boolean working;

    //-----------------------------------------BankleitzahlenVerzeichnisDialog--

}

/**
 * Model of the {@code BankleitzahlenVerzeichnisDialog}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: BankleitzahlenVerzeichnisDialog.java 3168 2007-04-28 03:30:51Z schulte2005 $
 */
class BankleitzahlenVerzeichnisDialogModel implements TableModel
{

    //--TableModel--------------------------------------------------------------

    /**
     * {@code TableModelEvent} listener list.
     * @serial
     */
    private EventListenerList tableListeners = new EventListenerList();

    public int getRowCount()
    {
        return this.getEntities().size();
    }

    public int getColumnCount()
    {
        return 5;
    }

    public String getColumnName(int columnIndex)
    {
        switch(columnIndex)
        {
            case 0: return BankleitzahlenVerzeichnisDialogBundle.
                getBankleitzahlColumnText(Locale.getDefault());

            case 1: return BankleitzahlenVerzeichnisDialogBundle.
                getBanknameColumnText(Locale.getDefault());

            case 2: return BankleitzahlenVerzeichnisDialogBundle.
                getPostalCodeColumnText(Locale.getDefault());

            case 3: return BankleitzahlenVerzeichnisDialogBundle.
                getCityColumnText(Locale.getDefault());

            case 4: return BankleitzahlenVerzeichnisDialogBundle.
                getBicColumnText(Locale.getDefault());

            default: return null;

        }
    }

    public Class<?> getColumnClass(int columnIndex)
    {
        return String.class;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return false;
    }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
        if(rowIndex >= this.getEntities().size())
        {
            throw new IllegalArgumentException(Integer.toString(rowIndex));
        }
        if(columnIndex >= this.getColumnCount())
        {
            throw new IllegalArgumentException(Integer.toString(columnIndex));
        }

        final BankleitzahlInfo info = this.getEntities().get(rowIndex);
        switch(columnIndex)
        {
            case 0: return info.getBankCode().
                format(Bankleitzahl.LETTER_FORMAT);

            case 1: return info.getName();

            case 2: return new DecimalFormat("#####").
                format(info.getPostalCode());

            case 3: return info.getCity();

            case 4: return info.getBic();

            default: return info.toString();

        }
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
    }

    public final void addTableModelListener(final TableModelListener l)
    {
        this.tableListeners.add(TableModelListener.class, l);
    }

    public final void removeTableModelListener(final TableModelListener l)
    {
        this.tableListeners.remove(TableModelListener.class, l);
    }

    //--------------------------------------------------------------TableModel--
    //--BankleitzahlenVerzeichnisDialogModel------------------------------------

    /**
     * Collection of {@code BankleitzahlInfo} instances.
     * @serial
     */
    private ArrayList<BankleitzahlInfo> entities;

    /**
     * Sorts {@code BankleitzahlInfo} instances by properties {@code city} and
     * {@code name}.
     */
    public static Comparator BANKLEITZAHLINFO_SORTER = new Comparator()
    {
        public int compare(final Object o1, final Object o2)
        {
            if(!(o1 instanceof BankleitzahlInfo))
            {
                throw new IllegalArgumentException(o1.getClass().getName());
            }
            if(!(o2 instanceof BankleitzahlInfo))
            {
                throw new IllegalArgumentException(o2.getClass().getName());
            }

            final BankleitzahlInfo b1 = (BankleitzahlInfo) o1;
            final BankleitzahlInfo b2 = (BankleitzahlInfo) o2;

            int ret = b1.getCity().compareTo(b2.getCity());

            if(ret == 0)
            {
                ret = b1.getName().compareTo(b2.getName());
            }

            return ret;
        }
    };

    /**
     * Gets the entities of the model.
     *
     * @return all entities of the model.
     */
    public ArrayList<BankleitzahlInfo> getEntities()
    {
        if(this.entities == null)
        {
            this.entities = new ArrayList(2000);
        }

        return this.entities;
    }

    protected void add(final BankleitzahlInfo info)
    {
        if(info == null)
        {
            throw new NullPointerException("info");
        }

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                getEntities().add(info);
                Collections.sort(getEntities(), BANKLEITZAHLINFO_SORTER);
                fireTableChanged(new TableModelEvent(
                    BankleitzahlenVerzeichnisDialogModel.this));

            }
        });
    }

    protected void addAll(final Collection<BankleitzahlInfo> infos)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                if(infos != null)
                {
                    getEntities().ensureCapacity(infos.size());
                    getEntities().addAll(infos);
                }
                Collections.sort(getEntities(), BANKLEITZAHLINFO_SORTER);
                fireTableChanged(new TableModelEvent(
                    BankleitzahlenVerzeichnisDialogModel.this));

            }
        });
    }

    protected void remove(final BankleitzahlInfo info)
    {
        if(info == null)
        {
            throw new NullPointerException("info");
        }

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final int row = getEntities().indexOf(info);
                if(row != -1)
                {
                    getEntities().remove(info);
                    fireTableChanged(new TableModelEvent(
                        BankleitzahlenVerzeichnisDialogModel.this, row, row,
                        TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));

                }
            }
        });
    }

    protected void clear()
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                getEntities().clear();
                fireTableChanged(new TableModelEvent(
                    BankleitzahlenVerzeichnisDialogModel.this));

            }
        });
    }


    /**
     * Notifies all registered {@code TableModelListener}s about {@code event}.
     *
     * @param event the event for the listeners.
     */
    protected void fireTableChanged(final TableModelEvent event)
    {
        if(event != null)
        {
            final Object[] listeners = this.tableListeners.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2)
            {
                if (listeners[i] == TableModelListener.class)
                {
                    ((TableModelListener) listeners[i + 1]).tableChanged(event);
                }
            }
        }
    }

    //------------------------------------BankleitzahlenVerzeichnisDialogModel--
}
